@@ -35,6 +35,9 @@ omxBuildAutoStartModel <- function(model, type=c('ULS', 'DWLS')) {
 	if(imxHasDefinitionVariable(model)){
 		stop("Definition variables found. Automatic start values are not implemented for models with definition variables.\nNo plans have been made to add these soon.")
 	}
+	if(imxIsMultilevel(model)){
+		stop("Multilevel model found. Automatic start values are not implemented for multilevel models.\nNo plans have been made to add these soon.")
+	}
 
 	isMultiGroupModel <- is.null(model$expectation) && (class(model$fitfunction) %in% "MxFitFunctionMultigroup")
 
@@ -62,13 +65,13 @@ omxBuildAutoStartModel <- function(model, type=c('ULS', 'DWLS')) {
 ##' This function automatically picks very good starting values for many models (RAM, LISREL, Normal), including multiple group versions of these.
 ##' It works for models with algebras. Models of continuous, ordinal, and joint ordinal-continuous variables are also acceptable.
 ##' It works for models with covariance or raw data.
-##' However, it does not currently work for models with definition variables, state space models, and item factor analysis models.
+##' However, it does not currently work for models with definition variables, state space models, item factor analysis models, or multilevel models; it only works well for GREML models under certain circumstances (see further below).
 ##'
 ##' The method used to obtain new starting values is quite simple. The user's model is changed to an unweighted least squares (ULS) model. The ULS model is estimated and its final point estimates are returned as the new starting values. Optionally, diagonally weighted least squares (DWLS) can be used instead with the \code{type} argument.
 ##'
 ##' Please note that ULS is sensitive to the scales of your variables. For example, if you have variables with means of 20 and variances of 0.001, then ULS will "weight" the means 20,000 times more than the variances and might result in zero variance estimates. Likewise if one variable has a variance of 20 and another has a variance of 0.001, the same problem may arise. To avoid this, make sure your variables are scaled accordingly. You could also use \code{type='DWLS'} to have the function use diagonally weighted least squares to obtain starting values.  Of course, using diagonally weighted least squares will take much much longer and will usually not provide better starting values than unweighted least squares.
 ##'
-##' Also note that if \code{model} contains a \link[=mxExpectationGREML]{GREML expectation}, argument \code{type} is ignored, and the function always uses a form of ULS.
+##' Also note that if \code{model} contains a \link[=mxExpectationGREML]{GREML expectation}, argument \code{type} is ignored, and the function always uses a form of ULS.  The function can be very helpful for finding good starting values for GREML MxModels that use an "implicit" (i.e., "specified in terms of covariates via \code{\link{mxExpectationGREML}()} argument \code{Xvars}") model for the phenotypic mean.  In contrast, the function is not generally recommended for GREML MxModels that use an "explicit" (i.e., "specified in terms of an MxMatrix or MxAlgebra via argument \code{yhat}") model for the phenotypic mean, and indeed, it can be downright counterproductive in such cases.
 ##'
 ##' @return
 ##' an MxModel with new free parameter values
@@ -161,55 +164,144 @@ autoStartDataHelper <- function(model, subname=model@name, type){
 
 
 autoStartGREML <- function(model){
-	if(model$expectation$dataset.is.yX){
-		y <- model$data$observed[,1]
-		X <- model$data$observed[,-1]
-		casesToDrop <- model$expectation$casesToDrop
-	} else{
-		dat <- mxGREMLDataHandler(
-			data=model$data$observed,
-			yvars=model$expectation$yvars,
-			Xvars=model$expectation$Xvars,
-			addOnes=model$expectation$addOnes,
-			blockByPheno=model$expectation$blockByPheno,
-			staggerZeroes=model$expectation$staggerZeroes
-		)
+	if(!model$expectation$REML && length(model$expectation$yhat) && !model$expectation$dataset.is.yX){
+		yhatdims <- dim(mxEvalByName(model$expectation$yhat,model,T))
+		if( !(yhatdims[1]>=1 && yhatdims[2]==1) ){
+			stop("to use `mxAutoStart()` with a GREML expectation that has an explicit means model, 'yhat' must have 1 column and at least 1 row")
+		}
+		Vdims <- dim(mxEvalByName(model$expectation$V,model,T))
+		if(Vdims[1]!=Vdims[2]){
+			stop("'V' must be a square matrix")
+		}
+		warning(paste0(
+			"using `mxAutoStart()` with a GREML expectation that has an explicit means model does not work very well,\n",
+			"and may even be counterproductive"))
+		dat <- mxGREMLDataHandler(data=model$data$observed,yvars=model$expectation$yvars)
 		y <- dat$yX[,1]
-		X <- dat$yX[,-1]
+		ly <- length(y)
 		casesToDrop <- dat$casesToDrop
 		rm(dat)
+		# Create variable bindings so CRAN check doesn't complain about mxAlgebra expressions
+		filtv <- NULL
+		filtm <- NULL
+		yyT <- NULL
+		V <- NULL
+		yhat <- NULL
+		OKflag <- FALSE
+		if(yhatdims[1]==ly && Vdims[1]==ly){
+			filt_mtx_v <- mxMatrix(type="Full",nrow=1,ncol=1,free=F,values=1,name="filtv")
+			filt_mtx_m <- mxMatrix(type="Full",nrow=1,ncol=1,free=F,values=1,name="filtm")
+			aff <- mxAlgebra( sum( ( vech(yyT) - vech(V + (yhat%*%t(yhat))) )%^%2 ), name="algfitfunc" )
+			OKflag <- TRUE
+		}
+		if(yhatdims[1]==ly && Vdims[1]>ly){
+			filt_mtx_v <- mxMatrix(type="Full",nrow=1,ncol=Vdims[1],free=F,values=1,name="filtv",condenseSlots=T)
+			filt_mtx_v@values[casesToDrop] <- 0
+			filt_mtx_m <- mxMatrix(type="Full",nrow=1,ncol=1,free=F,values=1,name="filtm")
+			aff <- mxAlgebra( sum( ( vech(yyT) - vech(omxSelectRowsAndCols(V,filtv) + (yhat%*%t(yhat))) )%^%2 ), name="algfitfunc" )
+			OKflag <- TRUE
+		}
+		if(yhatdims[1]>ly && Vdims[1]==ly){
+			filt_mtx_m <- mxMatrix(type="Full",nrow=1,ncol=yhatdims[1],free=F,values=1,name="filtm",condenseSlots=T)
+			filt_mtx_m@values[casesToDrop] <- 0
+			filt_mtx_v <- mxMatrix(type="Full",nrow=1,ncol=1,free=F,values=1,name="filtv")
+			aff <- mxAlgebra( sum( ( vech(yyT) - vech( V + omxSelectRowsAndCols(yhat%*%t(yhat),filtm) ) )%^%2 ), name="algfitfunc" )
+			OKflag <- TRUE
+		}
+		if(yhatdims[1]>ly && Vdims[1]>ly){
+			filt_mtx_v <- mxMatrix(type="Full",nrow=1,ncol=Vdims[1],free=F,values=1,name="filtv",condenseSlots=T)
+			filt_mtx_v@values[casesToDrop] <- 0
+			filt_mtx_m <- mxMatrix(type="Full",nrow=1,ncol=yhatdims[1],free=F,values=1,name="filtm",condenseSlots=T)
+			filt_mtx_m@values[casesToDrop] <- 0
+			aff <- mxAlgebra( sum( ( vech(yyT) - vech(omxSelectRowsAndCols(V,filtv) + omxSelectRowsAndCols(yhat%*%t(yhat),filtm)) )%^%2 ), name="algfitfunc" )
+			OKflag <- TRUE
+		}
+		if(!OKflag){
+			stop(
+				paste0(
+					"something's wrong;\n",
+					"check the dimensions of 'V', 'yhat', and phenotype vector 'y'\n",
+					"(use `mxGREMLDataHandler()` to construct 'y' prior to runtime)"
+				)
+			)
+		}
+		tempmod <- mxModel(
+			"tmp",
+			model,
+			aff,
+			filt_mtx_v,
+			filt_mtx_m,
+			mxMatrix(type="Symm",nrow=ly,free=F,values=outer(y,y),name="yyT",condenseSlots=T),
+			mxAlgebraFromString(paste(model$name,model$expectation$V,sep="."),name="V"),
+			mxAlgebraFromString(paste(model$name,model$expectation$yhat,sep="."),name="yhat"),
+			mxFitFunctionAlgebra("algfitfunc")
+		)
+		rm(y,filt_mtx_v,filt_mtx_m,aff)
+		tempmod <- mxOption(tempmod,"Calculate Hessian","No")
+		tempmod <- mxOption(tempmod,"Standard Errors","No")
+		tempmod <- mxRun(tempmod,silent=T)
+		newparams <- coef(tempmod)
+		rm(tempmod)
+		oldparams <- coef(model)
+		model <- omxSetParameters(model, values=newparams, labels=names(oldparams))
+		return(model)
 	}
-	olsresids <- lm(y~X+0)$residuals
-	rm(X,y)
-	Vdim <- nrow(mxEvalByName(model$expectation$V,model,T))
-  # Create variable bindings so CRAN check doesn't complain about mxAlgebra expressions
-  filt <- NULL
-  S <- NULL
-  V <- NULL
-	if(length(olsresids) < Vdim){
-		filt_mtx <- mxMatrix(type="Full",nrow=1,ncol=Vdim,free=F,values=1,name="filt")
-		filt_mtx@values[casesToDrop] <- 0
-		aff <- mxAlgebra( sum((vech(S) - vech(omxSelectRowsAndCols(V,filt)))%^%2), name="algfitfunc")
-	} else{
-		filt_mtx <- mxMatrix(type="Full",nrow=1,ncol=1,free=F,values=1,name="filt")
-		aff <- mxAlgebra( sum((vech(S)-vech(V))%^%2), name="algfitfunc")
+	else{
+		if(model$expectation$dataset.is.yX){
+			y <- model$data$observed[,1]
+			X <- model$data$observed[,-1]
+			casesToDrop <- model$expectation$casesToDrop
+		} 
+		else{
+			dat <- mxGREMLDataHandler(
+				data=model$data$observed,
+				yvars=model$expectation$yvars,
+				Xvars=model$expectation$Xvars,
+				addOnes=model$expectation$addOnes,
+				blockByPheno=model$expectation$blockByPheno,
+				staggerZeroes=model$expectation$staggerZeroes
+			)
+			y <- dat$yX[,1]
+			X <- dat$yX[,-1]
+			casesToDrop <- dat$casesToDrop
+			rm(dat)
+		}
+		olsresids <- lm(y~X+0)$residuals
+		rm(X,y)
+		Vdim <- dim(mxEvalByName(model$expectation$V,model,T))
+		if(Vdim[1]!=Vdim[2]){
+			stop("'V' must be a square matrix")
+		}
+		Vdim <- Vdim[1]
+		# Create variable bindings so CRAN check doesn't complain about mxAlgebra expressions
+		filt <- NULL
+		S <- NULL
+		V <- NULL
+		if(length(olsresids) < Vdim){
+			filt_mtx <- mxMatrix(type="Full",nrow=1,ncol=Vdim,free=F,values=1,name="filt")
+			filt_mtx@values[casesToDrop] <- 0
+			aff <- mxAlgebra( sum((vech(S) - vech(omxSelectRowsAndCols(V,filt)))%^%2), name="algfitfunc")
+		} else{
+			filt_mtx <- mxMatrix(type="Full",nrow=1,ncol=1,free=F,values=1,name="filt")
+			aff <- mxAlgebra( sum((vech(S)-vech(V))%^%2), name="algfitfunc")
+		}
+		tempmod <- mxModel(
+			"tmp",
+			model,
+			aff,
+			filt_mtx,
+			mxMatrix(type="Symm",nrow=length(olsresids),free=F,values=outer(olsresids,olsresids),name="S",condenseSlots=T),
+			mxAlgebraFromString(paste(model$name,model$expectation$V,sep="."),name="V"),
+			mxFitFunctionAlgebra("algfitfunc")
+		)
+		rm(olsresids,aff,filt_mtx)
+		tempmod <- mxOption(tempmod,"Calculate Hessian","No")
+		tempmod <- mxOption(tempmod,"Standard Errors","No")
+		tempmod <- mxRun(tempmod,silent=T)
+		newparams <- coef(tempmod)
+		rm(tempmod)
+		oldparams <- coef(model)
+		model <- omxSetParameters(model, values=newparams, labels=names(oldparams))
+		return(model)
 	}
-	tempmod <- mxModel(
-		"tmp",
-		model,
-		aff,
-		filt_mtx,
-		mxMatrix(type="Symm",nrow=length(olsresids),free=F,values=outer(olsresids,olsresids),name="S",condenseSlots=T),
-		mxAlgebraFromString(paste(model$name,model$expectation$V,sep="."),name="V"),
-		mxFitFunctionAlgebra("algfitfunc")
-	)
-	rm(olsresids,aff,filt_mtx)
-	tempmod <- mxOption(tempmod,"Calculate Hessian","No")
-	tempmod <- mxOption(tempmod,"Standard Errors","No")
-	tempmod <- mxRun(tempmod,silent=T)
-	newparams <- coef(tempmod)
-	rm(tempmod)
-	oldparams <- coef(model)
-	model <- omxSetParameters(model, values=newparams, labels=names(oldparams))
-	return(model)
 }
