@@ -187,7 +187,7 @@ double GradientOptimizerContext::solFun(double *myPars, int* mode)
 	}
 
 	if (verbose >= 3) {
-		mxLog("fit %f (mode %d)", fc->getUnscaledFit(), *mode);
+		mxLog("fit %.12f (mode %d)", fc->getUnscaledFit(), *mode);
 	}
 
 	return fc->getUnscaledFit();
@@ -369,9 +369,9 @@ void omxComputeGD::computeImpl(FitContext *fc)
 			fc->wanted |= FF_COMPUTE_GRADIENT | FF_COMPUTE_HESSIAN;
 		}
 		fc->constraintFunVals = rf.constraintFunValsOut;
-                fc->LagrMultipliers = rf.LagrMultipliersOut;
-                fc->constraintJacobian = rf.constraintJacobianOut;
-                fc->LagrHessian = rf.LagrHessianOut;
+		fc->LagrMultipliers = rf.LagrMultipliersOut;
+		fc->constraintJacobian = rf.constraintJacobianOut;
+		fc->LagrHessian = rf.LagrHessianOut;
 		break;
         case OptEngine_NLOPT:
 		if (rf.maxMajorIterations == -1) rf.maxMajorIterations = Global->majorIterations;
